@@ -47,7 +47,7 @@ void uwsgi_request_body_seek(struct wsgi_request *wsgi_req, off_t pos) {
 
 	if (uwsgi.post_buffering) {
 		if (pos < 0) {
-			if (pos > (off_t) wsgi_req->post_pos) {
+			if ((off_t) wsgi_req->post_pos - pos < 0) {
 				wsgi_req->post_pos = 0;
 				return;
 			}
