@@ -607,6 +607,20 @@ void uwsgi_close_request(struct wsgi_request *wsgi_req) {
 		uwsgi.workers[uwsgi.mywid].rss_size = rss;
 	}
 
+	if (!wsgi_req->do_not_account) {
+		uwsgi.workers[0].requests++;
+		uwsgi.workers[uwsgi.mywid].requests++;
+		uwsgi.workers[uwsgi.mywid].cores[wsgi_req->async_id].requests++;
+		uwsgi.workers[uwsgi.mywid].cores[wsgi_req->async_id].write_errors += wsgi_req->write_errors;
+		// this is used for MAX_REQUESTS
+		uwsgi.workers[uwsgi.mywid].delta_requests++;
+	}
+
+#ifdef UWSGI_ROUTING
+	// apply final routes after accounting
+	uwsgi_apply_final_routes(wsgi_req);
+#endif
+
 	// close the connection with the client
 	if (!wsgi_req->fd_closed) {
 		// NOTE, if we close the socket before receiving eventually sent data, socket layer will send a RST
@@ -629,15 +643,6 @@ void uwsgi_close_request(struct wsgi_request *wsgi_req) {
 		free(wsgi_req->proto_parser_buf);
 	}
 
-	if (!wsgi_req->do_not_account) {
-		uwsgi.workers[0].requests++;
-		uwsgi.workers[uwsgi.mywid].requests++;
-		uwsgi.workers[uwsgi.mywid].cores[wsgi_req->async_id].requests++;
-		uwsgi.workers[uwsgi.mywid].cores[wsgi_req->async_id].write_errors += wsgi_req->write_errors;
-		// this is used for MAX_REQUESTS
-		uwsgi.workers[uwsgi.mywid].delta_requests++;
-	}
-
 	// after_request hook
 	if (uwsgi.p[wsgi_req->uh->modifier1]->after_request)
 		uwsgi.p[wsgi_req->uh->modifier1]->after_request(wsgi_req);
@@ -2225,6 +2230,7 @@ struct uwsgi_string_list *uwsgi_string_new_list(struct uwsgi_string_list **list,
 	uwsgi_string->next = NULL;
 	uwsgi_string->custom = 0;
 	uwsgi_string->custom2 = 0;
+	uwsgi_string->custom_ptr = NULL;
 
 	return uwsgi_string;
 }
@@ -3939,3 +3945,14 @@ void http_url_encode(char *buf, uint16_t *len, char *dst) {
 
 }
 
+void uwsgi_takeover() {
+	if (uwsgi.i_am_a_spooler) {
+		uwsgi_spooler_run();
+	}
+	else if (uwsgi.muleid) {
+		uwsgi_mule_run();
+	}
+	else {
+		uwsgi_worker_run();
+	}
+}
