@@ -154,9 +154,14 @@ XS(XS_coroae_accept_request) {
 
 request:
 
+#ifdef UWSGI_ROUTING
+        if (uwsgi_apply_routes(wsgi_req) == UWSGI_ROUTE_BREAK) {
+                goto end;
+        }
+#endif
+
         for(;;) {
-                wsgi_req->async_status = uwsgi.p[wsgi_req->uh->modifier1]->request(wsgi_req);
-                if (wsgi_req->async_status <= UWSGI_OK) {
+                if (uwsgi.p[wsgi_req->uh->modifier1]->request(wsgi_req)) {
                         goto end;
                 }
                 wsgi_req->switches++;
