@@ -282,7 +282,7 @@ apply:
                 return UWSGI_OK;
 	} 
 	output = uwsgi_xslt_apply(doc, stylesheet, params, &output_rlen);
-	xmlFree(doc);
+	xmlFreeDoc(doc);
 	if (params) free(params);
 	if (!output) {
 		uwsgi_500(wsgi_req);
@@ -316,9 +316,10 @@ static void uwsgi_xslt_log(struct wsgi_request *wsgi_req) {
 	log_request(wsgi_req);
 }
 
-static int transform_toxslt(struct wsgi_request *wsgi_req, struct uwsgi_buffer *ub, struct uwsgi_buffer **new, void *data) {
+static int transform_toxslt(struct wsgi_request *wsgi_req, struct uwsgi_transformation *ut) {
 	int ret = -1;
-        struct uwsgi_transformation_xslt_conf *utxc = (struct uwsgi_transformation_xslt_conf *) data;
+        struct uwsgi_transformation_xslt_conf *utxc = (struct uwsgi_transformation_xslt_conf *) ut->data;
+	struct uwsgi_buffer *ub = ut->chunk;
 
 	xmlDoc *doc = xmlReadMemory(ub->buf, ub->pos, NULL, NULL, 0);
 	if (!doc) goto end;
@@ -327,18 +328,12 @@ static int transform_toxslt(struct wsgi_request *wsgi_req, struct uwsgi_buffer *
         char *output = uwsgi_xslt_apply( doc, utxc->stylesheet->buf, utxc->params ? utxc->params->buf : NULL, &rlen);
 	if (!output) goto end;
 
-        if (uwsgi_response_prepare_headers_int(wsgi_req, wsgi_req->status)) goto end;
-        if (uwsgi_response_add_content_length(wsgi_req, rlen)) goto end;
-        if (uwsgi_response_add_content_type(wsgi_req, utxc->content_type->buf, utxc->content_type->pos)) goto end;
-
-	*new = uwsgi_buffer_new(rlen);
-	if (uwsgi_buffer_append(*new, output, rlen)) {
-		xmlFree(output);
-		uwsgi_buffer_destroy(*new);
-		*new = NULL;
-		goto end;
+	// do not check for errors !!!
+	if (ut->round == 1) {
+        	uwsgi_response_add_content_type(wsgi_req, utxc->content_type->buf, utxc->content_type->pos);
 	}
-	xmlFree(output);
+
+	uwsgi_buffer_map(ub, output, rlen);
 	ret = 0;
 
 end:
@@ -413,7 +408,7 @@ static int uwsgi_routing_func_xslt(struct wsgi_request *wsgi_req, struct uwsgi_r
 	xmlDoc *doc = xmlParseFile(ub_doc->buf) ;
 	if (!doc) goto end;
         char *output = uwsgi_xslt_apply(doc, ub_stylesheet->buf, ub_params ? ub_params->buf : NULL, &rlen);
-	xmlFree(doc);
+	xmlFreeDoc(doc);
 	if (!output) goto end;
 
         if (uwsgi_response_prepare_headers(wsgi_req, "200 OK", 6)) goto end;
